<?php

/**
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|L|o|g|i|n|b|a|s|i|c|o|E|m|o| Pro!
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 * @Author: emo
 * @Email: ipademofdz@gmail.com
 * @Date:   2024-07-22 12:34:56
 * @Last Modified by:   emo
 * @Last Modified time: 2024-08-12 14:10:40
 * website: https://loginbasicoemo.com
 *
 * Licensed under the MIT License.
 * You may obtain a copy of the license at
 * https://opensource.org/licenses/MIT
 *
 */

require APPPATH . '/libraries/REST_Controller.php';
require APPPATH . '/libraries/Format.php';
use Restserver\Libraries\REST_Controller;

/**
 * REST API
 */
class Api extends REST_Controller {
	public function __construct() {
		parent::__construct();

		$this->load->library('table');
		$this->load->model('User_model');

		// Obtener el idioma desde la configuración
		$default_language = $this->config->item('language');

		// Cargar el archivo de idioma
		$this->lang->load('aplicacion_lang', $default_language);

		// Verificar si la clase Form_validation ya está cargada
		if (!isset($this->form_validation)) {
			$this->load->library('form_validation');
		}
	}

	public function example_put() {
		//
	}
	public function example_delete() {
		//
	}

	/**
	 * AJAX APi POST
	 */
	public function ajax_post() {
		// Lista de tablas protegidas
		$protected_tables = array('archivo', 'config', 'user', 'user_access_menu', 'user_menu', 'user_role', 'user_sub_menu', 'user_token', 'keys');

		// Obtener valores de la solicitud, ya sea POST o GET
		$query_type = $this->input->get_post('query_type');
		$table = $this->input->get_post('table');
		$api = $this->input->get_post('api');

		// Validar que los parámetros requeridos están presentes
		if (!$query_type || !$table || empty($api)) {
			$this->log_error("Missing parameters: query_type, table, or api", $query_type, $table, $api);
			echo json_encode(array('error' => $this->lang->line('missing_params')));
			return;
		}

		// Validar la clave API
		if (!$this->validate_api_key($api)) {
			$this->log_error("Invalid API key: $api", $query_type, $table, $api);
			$response = array(
				'status' => 'error',
				'message' => 'Clave API inválida.',
			);
			echo json_encode($response);
			return;
		}

		// Verificar si la tabla está protegida
		if (in_array($table, $protected_tables)) {
			$this->log_error("Access denied to protected table: $table", $query_type, $table, $api);
			echo json_encode(array('error' => $this->lang->line('access_denied')));
			return;
		}

		// Validar que el nombre de la tabla es válido
		if (!preg_match('/^[a-zA-Z0-9_]+$/', $table)) {
			$this->log_error("Invalid table name: $table", $query_type, $table, $api);
			echo json_encode(array('error' => $this->lang->line('invalid_table_name')));
			return;
		}

		// Procesar la consulta según el tipo
		switch (strtoupper($query_type)) {
		case 'SELECT':
			$column = $this->input->get_post('column');
			$operator = $this->input->get_post('operator');
			$condition_value = $this->input->get_post('condition_value');
			$select_columns = $this->input->get_post('select_columns');

			// Validar parámetros requeridos para SELECT
			if (!$column || !$operator || !$condition_value) {
				$this->log_error("Missing parameters for SELECT: column, operator, or condition_value", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('missing_params_select')));
				return;
			}

			// Validar que el operador es uno de los permitidos
			$valid_operators = array('=', 'LIKE', '<', '>', '<=', '>=');
			if (!in_array($operator, $valid_operators)) {
				$this->log_error("Invalid operator for SELECT: $operator", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('invalid_operator')));
				return;
			}

			// Preparar la lista de columnas a seleccionar
			$columns_to_select = '*'; // valor por defecto
			if (!empty($select_columns)) {
				$columns_to_select = $this->db->protect_identifiers($select_columns, TRUE);
			}

			// Preparar y ejecutar la consulta SELECT
			$sql = "SELECT " . $columns_to_select . " FROM " . $this->db->protect_identifiers($table, TRUE) . " WHERE " . $this->db->protect_identifiers($column, TRUE) . " " . $operator . " ?";
			$query = $this->db->query($sql, array($condition_value));

			// Verificar si la consulta fue exitosa
			if ($query) {
				$results = $query->result_array();
				$this->log_success("SELECT query successful", $query_type, $table, $api, $column, $operator, $condition_value, '', '');
				echo json_encode($results);
			} else {
				$this->log_error("SELECT query error on table: $table", $query_type, $table, $api, $column, $operator, $condition_value);
				echo json_encode(array('error' => $this->lang->line('select_query_error')));
			}

			break;

		case 'INSERT':
			$fields = $this->input->get_post('fields');
			$values = $this->input->get_post('values');

			// Validar parámetros requeridos para INSERT
			if (!$fields || !$values) {
				$this->log_error("Missing parameters for INSERT: fields or values", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('missing_params_insert')));
				return;
			}

			// Separar y proteger los campos
			$fields_array = explode(',', $fields);
			foreach ($fields_array as $key => $field) {
				$fields_array[$key] = trim($field);
			}

			// Separar y limpiar los valores
			$values_array = explode(',', $values);
			foreach ($values_array as &$value) {
				$value = trim($value, "'");
			}

			// Verificar si el número de campos y valores coincide
			if (count($fields_array) !== count($values_array)) {
				$this->log_error("Fields and values mismatch for INSERT on table: $table", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('fields_values_mismatch')));
				return;
			}

			// Crear el arreglo asociativo de campos y valores
			$data = array();
			foreach ($fields_array as $index => $field) {
				$data[$field] = $values_array[$index];
			}

			// Insertar en la base de datos
			$query = $this->db->insert($table, $data);

			// Verificar si la consulta fue exitosa
			if ($query) {
				$this->log_success("INSERT query successful", $query_type, $table, $api, '', '', '', $fields, $values);
				echo json_encode(array('success' => 'true'));
			} else {
				$this->log_error("INSERT query error on table: $table", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('insert_error')));
			}

			break;

		case 'UPDATE':
			$column = $this->input->get_post('column');
			$operator = $this->input->get_post('operator');
			$condition_value = $this->input->get_post('condition_value');
			$fields = $this->input->get_post('fields');
			$values = $this->input->get_post('values');

			// Validar parámetros requeridos para UPDATE
			if (!$column || !$operator || !$condition_value || !$fields || !$values) {
				$this->log_error("Missing parameters for UPDATE: column, operator, condition_value, fields, or values", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('missing_params_update')));
				return;
			}

			// Separar y proteger los campos y valores
			$fields_array = explode(',', $fields);
			$values_array = explode(',', $values);
			if (count($fields_array) !== count($values_array)) {
				$this->log_error("Fields and values mismatch for UPDATE on table: $table", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('fields_values_mismatch')));
				return;
			}

			// Crear arreglo asociativo de campos y valores para UPDATE
			$update_data = array();
			foreach ($fields_array as $index => $field) {
				$update_data[trim($field)] = trim($values_array[$index], "'");
			}

			// Preparar y ejecutar la consulta UPDATE
			$this->db->set($update_data);
			$this->db->where($column . " " . $operator, $condition_value);

			// Verificar si la consulta fue exitosa
			$query = $this->db->update($table);
			if ($this->db->affected_rows() > 0) {
				$this->log_success("UPDATE query successful", $query_type, $table, $api, $column, $operator, $condition_value, $fields, $values);
				echo json_encode(array('success' => $this->lang->line('update_success')));
			} else {
				$this->log_error("UPDATE query error on table: $table", $query_type, $table, $api, $column, $operator, $condition_value);
				echo json_encode(array('error' => $this->lang->line('update_error')));
			}

			break;

		case 'DELETE':
			$column = $this->input->get_post('column');
			$operator = $this->input->get_post('operator');
			$condition_value = $this->input->get_post('condition_value');

			// Validar parámetros requeridos para DELETE
			if (!$column || !$operator || !$condition_value) {
				$this->log_error("Missing parameters for DELETE: column, operator, or condition_value", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('missing_params_delete')));
				return;
			}

			// Validar que el operador es uno de los permitidos
			$valid_operators = array('=', '<', '>', '<=', '>=');
			if (!in_array($operator, $valid_operators)) {
				$this->log_error("Invalid operator for DELETE: $operator", $query_type, $table, $api);
				echo json_encode(array('error' => $this->lang->line('invalid_operator')));
				return;
			}

			// Preparar y ejecutar la consulta DELETE
			$this->db->where($this->db->protect_identifiers($column, TRUE) . " " . $operator, $condition_value);
			$query = $this->db->delete($this->db->protect_identifiers($table, TRUE));

			// Verificar si la consulta fue exitosa
			if ($this->db->affected_rows() > 0) {
				$this->log_success("DELETE query successful", $query_type, $table, $api, $column, $operator, $condition_value, '', '');
				echo json_encode(array('success' => $this->lang->line('delete_success')));
			} else {
				$this->log_error("DELETE query error on table: $table", $query_type, $table, $api, $column, $operator, $condition_value);
				echo json_encode(array('error' => $this->lang->line('delete_error')));
			}

			break;

		default:
			$this->log_error("Unsupported query type: $query_type", $query_type, $table, $api);
			echo json_encode(array('error' => $this->lang->line('unsupported_query_type')));
			break;
		}
	}

	/**
	 * Logs an error.
	 *
	 * @param      <type>  $message          The message
	 * @param      <type>  $query_type       The query type
	 * @param      <type>  $table            The table
	 * @param      <type>  $api              The api
	 * @param      string  $column           The column
	 * @param      string  $operator         The operator
	 * @param      string  $condition_value  The condition value
	 * @param      string  $fields           The fields
	 * @param      string  $values           The values
	 */
	private function log_error($message, $query_type, $table, $api, $column = 'N/A', $operator = 'N/A', $condition_value = 'N/A', $fields = 'N/A', $values = 'N/A') {
		// Obtener la dirección IP del cliente
		$ip_address = $_SERVER['REMOTE_ADDR'];
		// Formatear el mensaje de log
		$log_message = sprintf(
			"[%s] IP: %s - %s - AJAX Request: Query Type: %s, Table: %s, Column: %s, Operator: %s, Condition Value: %s, Fields: %s, Values: %s, API: %s",
			date('Y-m-d H:i:s'),
			$ip_address,
			$message,
			$query_type,
			$table,
			$column,
			$operator,
			$condition_value,
			$fields,
			$values,
			$api
		);
		// Registrar el mensaje como error
		log_message('error', $log_message);
	}

	/**
	 * Logs a success.
	 *
	 * @param      <type>  $message          The message
	 * @param      <type>  $query_type       The query type
	 * @param      <type>  $table            The table
	 * @param      <type>  $api              The api
	 * @param      string  $column           The column
	 * @param      string  $operator         The operator
	 * @param      string  $condition_value  The condition value
	 * @param      string  $fields           The fields
	 * @param      string  $values           The values
	 */
	private function log_success($message, $query_type, $table, $api, $column = 'N/A', $operator = 'N/A', $condition_value = 'N/A', $fields = 'N/A', $values = 'N/A') {
		// Obtener la dirección IP del cliente
		$ip_address = $_SERVER['REMOTE_ADDR'];
		// Formatear el mensaje de log
		$log_message = sprintf(
			"[%s] IP: %s - %s - AJAX Request: Query Type: %s, Table: %s, Column: %s, Operator: %s, Condition Value: %s, Fields: %s, Values: %s, API: %s",
			date('Y-m-d H:i:s'),
			$ip_address,
			$message,
			$query_type,
			$table,
			$column,
			$operator,
			$condition_value,
			$fields,
			$values,
			$api
		);
		// Registrar el mensaje como éxito (debug)
		log_message('debug', $log_message);
	}

	/**
	 * Api List Post
	 */
	public function list_post() {

		// Lista de tablas protegidas
		$protected_tables = array('archivo', 'config', 'user', 'user_access_menu', 'user_menu', 'user_role', 'user_sub_menu', 'user_token', 'keys');

		$table = $this->input->get_post('table');
		$columns = $this->input->get_post('columns');
		$api = $this->input->get_post('api');

		// Verifica si se han pasado los parámetros necesarios
		if (empty($table) || empty($columns) || empty($api)) {
			// Devuelve un error en caso de que falten parámetros
			$response = array(
				'status' => 'error',
				'message' => 'Faltan parámetros necesarios.',
			);
			echo json_encode($response);
			return;
		}

		// Validar la clave API
		if (!$this->validate_api_key($api)) {
			$response = array(
				'status' => 'error',
				'message' => 'Clave API inválida.',
			);
			echo json_encode($response);
			return;
		}

		// Verificar si la tabla está protegida
		if (in_array($table, $protected_tables)) {
			echo json_encode(array('error' => $this->lang->line('access_denied')));
			return;
		}

		// Convierte los parámetros de columnas a un array
		$columns_array = explode(',', $columns);

		// Realiza la consulta a la base de datos
		$this->db->select($columns_array);
		$query = $this->db->get($table);

		// Establece las clases CSS para la tabla
		$template = array(
			'table_open' => '<table class="table">',
			'thead_open' => '<thead>',
			'thead_close' => '</thead>',
			'tbody_open' => '<tbody>',
			'tbody_close' => '</tbody>',
			'row_start' => '<tr>',
			'row_end' => '</tr>',
			'cell_start' => '<td>',
			'cell_end' => '</td>',
			'row_alt_start' => '<tr>',
			'row_alt_end' => '</tr>',
			'cell_alt_start' => '<td>',
			'cell_alt_end' => '</td>',
			'table_close' => '</table>',
		);

		$this->table->set_template($template);
		$this->table->set_heading($columns_array);
		$table_html = $this->table->generate($query);

		// Genera el HTML completo con los estilos incluidos
		$html = '<!DOCTYPE html>
<html>
<head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <style>
        .table {
            width: 100%;
            max-width: 100%;
            margin-bottom: 1rem;
            background-color: transparent;
            border-collapse: collapse;
        }

        .table th,
        .table td {
            padding: 0.75rem;
            vertical-align: top;
            border: 1px solid #dee2e6;
        }

        .table thead th {
            vertical-align: bottom;
            border-bottom: 2px solid #dee2e6;
            text-align: center; /* Centrar las cabeceras */
            background-color: rgba(255, 255, 255, 0.5); /* Color de fondo gris */
            font-size: 14px;
        }

        .table tbody td {
            color: #333; /* Color del texto de las celdas */
            font-size: 14px; /* Tamaño del texto de las celdas */
        }

        .table tbody tr:nth-of-type(odd) {
            background-color: rgba(255, 255, 255, 0.05); /* Color alternado */
        }

        .table-responsive {
            display: block;
            width: 100%;
            overflow-x: auto;
            -webkit-overflow-scrolling: touch;
            -ms-overflow-style: -ms-autohiding-scrollbar;
        }

        /* Estilo para filas al hacer hover */
        .table tbody tr:hover {
            background-color: #cce5ff; /* Cambia el color al hacer hover */
            cursor: pointer;
        }

        @media (max-width: 767.98px) {
            .table-responsive {
                border: 0;
            }
        }
    </style>
</head>
<body>
    <div style="height:20px;"></div>
    <div class="table-responsive" style="padding: 10px;">' . $table_html . '</div>
</body>
</html>';

		// Devuelve el HTML completo
		echo $html;
	}

	// Función para validar la clave API

	/**
	 * { function_description }
	 *
	 * @param      <type>  $api    The api
	 *
	 * @return     bool    ( description_of_the_return_value )
	 */
	public function validate_api_key($api) {
		// Lógica para validar la clave API
		if (empty($api)) {
			return false;
		}

		// Consulta a la base de datos para verificar la clave API
		$this->load->database();
		$query = $this->db->get_where('keys', ['key' => $api, 'is_active' => 1]);
		return $query->num_rows() > 0;
	}

//para plugins

	// Método para obtener todos los plugins de la base de datos (GET)

	/**
	 * Get Plugins from http get
	 */
	public function plugins_get() {
		log_message('debug', 'plugins_get method called');
		$plugins = $this->get_plugins();
		log_message('debug', 'Plugins retrieved: ' . print_r($plugins, true));
		$this->response($plugins, REST_Controller::HTTP_OK);
	}

	// Método para obtener un plugin específico por ID (GET)

	/**
	 * Get Plugins from http get by id
	 *
	 * @param      <type>  $id     The identifier
	 */
	public function plugin_get($id) {
		$plugin = $this->get_plugin_by_id($id);
		if ($plugin) {
			$this->response($plugin, REST_Controller::HTTP_OK);
		} else {
			$this->response(['error' => 'Plugin not found'], REST_Controller::HTTP_NOT_FOUND);
		}
	}

	// Métodos privados para interactuar con la base de datos

	/**
	 * Gets the plugins from table
	 *
	 * @return     <type>  The plugins.
	 */
	private function get_plugins() {
		$query = $this->db->get('plugins');
		return $query->result();
	}

	/**
	 * Gets the plugin by identifier from table
	 *
	 * @param      <type>  $id     The identifier
	 *
	 * @return     <type>  The plugin by identifier.
	 */
	private function get_plugin_by_id($id) {
		$query = $this->db->get_where('plugins', array('id' => $id));
		return $query->row();
	}
/*
public function save_plugins_post() {
$plugins = $this->input->raw_input_stream; // Obtener los datos crudos enviados en el POST
$plugins = json_decode($plugins, true); // Decodificar el JSON

if ($plugins) {
// Limpiar la tabla de plugins antes de guardar los nuevos
//$this->db->truncate('plugins');

// Insertar cada plugin en la base de datos
foreach ($plugins as $plugin) {
$data = [
'id' => $plugin['id'],
'name' => $plugin['name'],
'version' => $plugin['version'],
'available_version' => isset($plugin['available_version']) ? $plugin['available_version'] : null,
'description' => $plugin['description'],
'author' => $plugin['author'],
'download_link' => $plugin['download_link'],
];
$this->db->insert('plugins', $data);
}

// Responder con éxito
$this->output->set_content_type('application/json')->set_output(json_encode(['status' => 'success']));
} else {
// Responder con error si no se recibieron datos
$this->output->set_content_type('application/json')->set_output(json_encode(['status' => 'error', 'message' => 'No plugins received']));
}
}*/

	public function save_plugins_post() {
		$plugins = $this->input->raw_input_stream; // Obtener los datos crudos enviados en el POST
		$plugins = json_decode($plugins, true); // Decodificar el JSON

		if ($plugins) {
			// Marcar todos los plugins como descontinuados temporalmente
			$this->db->set('is_discontinued', 1)->update('plugins');

			// Insertar o actualizar cada plugin recibido desde el servidor central
			foreach ($plugins as $plugin) {
				$data = [
					'id' => $plugin['id'],
					'name' => $plugin['name'],
					'version' => $plugin['version'],
					'available_version' => isset($plugin['available_version']) ? $plugin['available_version'] : null,
					'description' => $plugin['description'],
					'author' => $plugin['author'],
					'download_link' => $plugin['download_link'],
					'is_discontinued' => 0, // Este plugin no está descontinuado
				];

				// Intentar actualizar el plugin existente
				$this->db->where('id', $plugin['id']);
				$this->db->update('plugins', $data);

				// Si no se actualizó ninguna fila, significa que el plugin no existe, por lo que se inserta
				if ($this->db->affected_rows() == 0) {
					$this->db->insert('plugins', $data);
				}
			}

			// Responder con éxito
			$this->output->set_content_type('application/json')->set_output(json_encode(['status' => 'success']));
		} else {
			// Responder con error si no se recibieron datos
			$this->output->set_content_type('application/json')->set_output(json_encode(['status' => 'error', 'message' => 'No plugins received']));
		}
	}

}

?>